package allgemein.core;

import java.awt.EventQueue;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

public class Calculator implements Calculatable
{
	private LinkedList<int[][]> pairs;
	private int K1len;
	private int K2len;
	private int K1lenEnd;
	private int K2lenEnd;
	
	private int Plen;

	private int Clen;

	private Iterator<Work> itr;   
	private int started=0;
	private boolean verbose;
	
	private int max_missing=4;
	private boolean searchRange=false;
	private int pos=0;
	private int max=0;
	private int round=0;
	private LinkedList<CalculatorResult> results;
	private LinkedList<Work> worklist;
	private PairGenerator pairgenerator;
	private ArrayList<CalculatorListener> listener;
	private CalculatorState st;
	
	
	
	
	public static void main(String[] args)
	{

		String plaintext="HALLODASHIERISTEINLANGERBEISPIELTEXTUMDASVERFAHRENZUZEIGEN";
		String chiffretext="NRSGSESAIEOZRABINADIILURTNDEHXUSRHEVIEEPAEHEEGTLZFTLIANMEL";
		
		long alt = System.currentTimeMillis();
		PairGenerator pg = new PairGenerator(plaintext,chiffretext,0);
		Calculator c = new Calculator(pg);
		c.setMiss(1);
		
		c.setK1len(8);
		c.setK2len(6);
		
		KeyList k = c.calculate();
		System.out.println(k);
		System.out.println( ((double)System.currentTimeMillis() -(double)alt )/1000+"s");
		
	}
	
	
	
	
	
	
	
	public Calculator(PairGenerator pg)
	{
		this.Plen = pg.getPlen();
		this.Clen = pg.getClen();
		this.K1len=0;
		this.K2len=0;
		this.pairgenerator = pg;
		
		listener=new ArrayList<CalculatorListener>();
		
	}

	public void addListener(CalculatorListener e)
	{
		listener.add(e);
	}
	
	private void stateChanged(CalculatorState s)
	{
		for (CalculatorListener l : listener)
		{
			EventQueue.invokeLater(new UpdateOrder(l,s));
		}
	}
	
	
	public int getK1len() {
		return K1len;
	}


	public void setK1len(int k1len) {
		K1len = k1len;
	}


	public int getK2len() {
		return K2len;
	}


	public void setK2len(int k2len) {
		K2len = k2len;
	}
	
	

	public boolean isSearchRange() {
		return searchRange;
	}


	public void setSearchRange(boolean searchRange) {
		this.searchRange = searchRange;
	}


	public synchronized KeyList calculate()
	{
		
		if (K1len ==0 || K2len==0 || this.max_missing<0 || this.max_missing>30)
		{
			st = new CalculatorState(CalculatorState.INVALID_KEYLENGTH);
			stateChanged(st);
			return null;
		}
		
		if (searchRange)
		{
			if (K1lenEnd < K1len || K2lenEnd < K2len)
			{
				st = new CalculatorState(CalculatorState.INVALID_RANGE);
				stateChanged(st);
				return null;
			}
			
			KeyList res = new KeyList();
			int startK1 = K1len;
			int startK2 = K2len;
			
			for (int i=startK1;i<=K1lenEnd;i++)
			{
				for (int j=startK2;j<=K2lenEnd;j++)
				{
					
					st = new CalculatorState(CalculatorState.NEW_KEYLENGTH_SEARCH_ROUND);
					st.addData("K1", i);
					st.addData("K2", j);
					stateChanged(st);
					
					this.K1len = i;
					this.K2len = j;
					KeyList r = this.calculateNow();
					if (r!= null)
						res.append(r);
				}
			}
			return res;
		}
		else
		{
			return calculateNow();
		}
	}

	private synchronized KeyList calculateNow()
	{		
		results = new LinkedList<CalculatorResult>();
		worklist = new LinkedList<Work>();
		pairgenerator.reset();
		verbose=false;

		for (round=1;round<80;round++)
		{
			if (verbose) System.out.println("Runde: " + (round));
			
			st = new CalculatorState(CalculatorState.NEW_ROUND);
			st.addData("ROUND", round);
			stateChanged(st);
			
			
			if (results.size()==0)
			{
				//Erster Durchlauf
				if (verbose) System.out.println("Noch keine Ergebnisse - berechne Paare");
				this.pairs = pairgenerator.getNextPairs();
				for (int[][] pa : pairs)
				{
					worklist.add(new Work(pa, Clen,K1len, K2len));
				}
				
			}
			else
			{
				LinkedList<int[][]> res = pairgenerator.getNextPairs();
				//Bereits Durchlufe 
				for (CalculatorResult r : results)
				{
					for (int[][] pa : res)
					{
						worklist.add(new Work(pa, Clen,K1len, K2len,r.getKeylist()));
					}
					
				}
			}
			
			st = new CalculatorState(CalculatorState.WORKLIST_CREATED);
			st.addData("SIZE", worklist.size());
			stateChanged(st);
			
			
			if (worklist.size()==0)
			{
				if (results.size()>0)
				{
					KeyList k = new KeyList();
					for (CalculatorResult r : results)
					{
						k.append(r.getKeylist());
					}
					
					st = new CalculatorState(CalculatorState.CALCULATION_FINISHED_SUCCESS);
					st.addData("KEYLIST", k);
					st.addData("K1", K1len);
					st.addData("K2", K2len);
					stateChanged(st);
					
					return k;
				} else {
					
					st = new CalculatorState(CalculatorState.CALCULATION_FINISHED_NORESULTS);
					st.addData("K1", K1len);
					st.addData("K2", K2len);
					stateChanged(st);
					
					return null;
				}
			}
			
			
			results = new LinkedList<CalculatorResult>();
			
			if (this.pairs==null)
			{
				st = new CalculatorState(CalculatorState.CALCULATION_FINISHED_NOPAIRS);
				stateChanged(st);
				
				return null;
			}
			
			itr = worklist.iterator();
			pos=0;
			max=worklist.size();
			
			st = new CalculatorState(CalculatorState.CALCULATION_THREADS_START);
			stateChanged(st);
			for (int i=1;i<=5;i++)
			{
				new CalculatorThread(i,this);
				started++;
			}
			
			
			
			try {
				this.wait();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			st = new CalculatorState(CalculatorState.CALCULATION_THREADS_STOP);
			stateChanged(st);
			
			//alle Threads sind beendet
			this.pairs.clear();
			int x=0;
			
			for (CalculatorResult r : results)
			{
				x+= r.getKeylist().size();
				if (r.getKeylist().size()==1)
				{
					if (verbose)System.out.println("Missing: " + r.getKeylist().get(0).missing());
				}
				
				//System.out.println(r.getKeylist());
			}
			if (verbose) System.out.println("Runde " + round + " ergab " + x + " mgliche Schlssel");
			
			st = new CalculatorState(CalculatorState.CALCULATION_RESULTENTRIES);
			st.addData("COUNT", results.size());
			st.addData("KEYS", x);
			stateChanged(st);
			
			
			if (results.size()==1)
			{
				KeyList k = results.get(0).getKeylist();
				if (k.size()==1)
					if (k.get(0).missing()<=max_missing)
					{
						if (k.get(0).klist.size()==1)
						{
							if (k.get(0).klist.get(0).missing()<=max_missing)
							{
								st = new CalculatorState(CalculatorState.CALCULATION_FINISHED_SUCCESS);
								st.addData("KEYLIST",k);
								st.addData("K1", K1len);
								st.addData("K2", K2len);
								stateChanged(st);
								
								return results.get(0).getKeylist();
							}
						}
					}
			}
			if (results.size()==0)
			{
				st = new CalculatorState(CalculatorState.CALCULATION_FINISHED_NORESULTS);
				st.addData("K1", K1len);
				st.addData("K2", K2len);
				stateChanged(st);
				
				return null;
			}
				
	
		}
		
		return null;
		
	}
	
	
	@Override
	public synchronized Work getWork() {
		// TODO Auto-generated method stub
		
		if (itr.hasNext())
		{
			Work w = itr.next();
			itr.remove();
			return w;
		}
		
		return null;
	}

	@Override
	public synchronized void setResult(int id, Work w, KeyList k) {
		// TODO Auto-generated method stub
		
		pos +=1;
		
		
		if (pos%100==0)
		{
			st = new CalculatorState(CalculatorState.CALCULATION_NEW_RESULT);
			st.addData("POS", pos);
			st.addData("MAX", max);
			st.addData("ROUND", round);
			stateChanged(st);
		}
		
		if (k.size()>=1)
		{
			results.add(new CalculatorResult(w,k));
		}
	
	}

	@Override
	public synchronized void done(int id) {
		// TODO Auto-generated method stub
		//System.out.println("Thread"+id+" finished");
		started--;
		if (started==0)
			allDone();
	}
	
	public synchronized void allDone()
	{
		worklist.clear();
		notifyAll();
	}


	public int getK1lenEnd() {
		return K1lenEnd;
	}


	public void setK1lenEnd(int k1lenEnd) {
		K1lenEnd = k1lenEnd;
	}


	public int getK2lenEnd() {
		return K2lenEnd;
	}


	public void setK2lenEnd(int k2lenEnd) {
		K2lenEnd = k2lenEnd;
	}
	
	public void setMiss(int miss)
	{
		this.max_missing=miss;
	}
	
	public int getPlen() {
		return Plen;
	}


	public void setPlen(int plen) {
		Plen = plen;
	}


	public int getClen() {
		return Clen;
	}


	public void setClen(int clen) {
		Clen = clen;
	}


}
