package allgemein.core;

import java.util.LinkedList;
import java.util.TreeMap;

public class PairGenerator {
	private char[] klartext;
	private char[] chiffretext;

	
	private char[] kList;
	private int[] posList;
	private int delta;
	
	private TreeMap<Integer, String> plainFreq;
	private TreeMap<Integer, String> ciphFreq;
	private TreeMap<Character, Integer> map;
	
	private int pointer = 0;

	private String ciphFreqString;
	private int count=1;
	
	public PairGenerator(String klar, String ciph, int delta) {
		this.klartext = klar.toCharArray();
		this.chiffretext = ciph.toCharArray();

		plainFreq = new TreeMap<Integer, String>();
		ciphFreq = new TreeMap<Integer, String>();
		
		this.delta=delta;
		kList = new char[klartext.length+1];
		posList = new int[klartext.length+1];

		map = new TreeMap<Character, Integer>();
		plainFreq = new TreeMap<Integer, String>();


		/*
		 * Hufigkeitsanalyse ber Klartext starten
		 */
		for (char c : this.klartext) {
			Integer i = map.get(c);
			if (i == null)
				i = 0;
			map.put(c, ++i);
		}

	  /*
	   * Aus selten vorkommenden Zeichen 
	   * einen String zusammensetzen
	   */
		for (char c : map.keySet()) {
			Integer i = map.get(c);
			String s = plainFreq.get(i);
			if (s == null)
				s = "";
			plainFreq.put(i, s + c);
		}

		map.clear();

		/*
		 * Hufigkeitsanalyse ber Chiffretext starten
		 */
		for (char c : this.chiffretext) {
			Integer i = map.get(c);
			if (i == null)
				i = 0;
			map.put(c, ++i);
		}


		ciphFreqString="";
		for (char c : map.keySet()) {
			Integer i = map.get(c);
			if (i==1) count++;
			String s = ciphFreq.get(i);
			if (s == null)
				s = "";
			ciphFreq.put(i, s + c);
		}
		
	  /*
	   * Aus selten vorkommenden Zeichen 
	   * einen String zusammensetzen
	   */
		for (Integer anz : ciphFreq.keySet())  
		{
			ciphFreqString += ciphFreq.get(anz); 
		}	
		
		int j=0;
		/*
		 * ciphFreqString enthlt eine sortierte
		 * Folge von selten vorkommende Zeichen
		 */
		for (char c : ciphFreqString.toCharArray()) 
		{
			for (int i=0;i<klartext.length;i++)
			{
				/*
				 * Falls das Klartextzeichen ein 
				 * seltenens Zeichen ist ...
				 */
				if (klartext[i]==c)
				{
					/*
					 * ... wird das Feld posList mit der
					 * Position dieses Zeichens befllt
					 */
					kList[j]=c;
					posList[j]=delta+i;
					j=j+1;
				}
			}
		}
		posList[j]=-1;
		
	}

	public static void main(String[] args) {
		String plaintext = "HALLODASHIERISTEINLANGERBEISPIELTEXTUMDASVERFAHRENZUZEIGEN";
		String chiffretext = "NRSGSESAIEOZRABINADIILURTNDEHXUSRHEVIEEPAEHEEGTLZFTLIANMEL";
		
		PairGenerator p = new PairGenerator(plaintext, chiffretext,0);
		LinkedList<int[][]> res = p.getNextPairs();

		for (int[][] x : res) {
			for (int i = 0; i < x.length; i++) {
				System.out.print(x[i][0] + "/" + x[i][1] + " ");
			}
			System.out.println();
		}

		res = p.getNextPairs();

		for (int[][] x : res) {
			for (int i = 0; i < x.length; i++) {
				System.out.print(x[i][0] + "/" + x[i][1] + " ");
			}
			System.out.println();
		}

		System.out.println(res.size());

	}

	public LinkedList<int[][]> getNextPairs() 
	{
		/* Ergebnisliste*/
		LinkedList<int[][]> result = new LinkedList<int[][]>();
		int anz=1;
		
		/*
		 * Beim ersten Durchlauf werden 
		 * sofort mehrere Paar gebildet
		 */
		if (pointer==0) anz=3; 
		
		for (int z=0;z<anz;z++)
		{
			/*
			 * posList enthlt die Position eines Zeichens
			 * im Klartext. Diese Liste ist so sortiert, das
			 * selten vorkommende Zeichen am Anfang stehen.
			 */
			if (pointer < kList.length && posList[pointer]!=-1)
			{
				char c=kList[pointer];
				LinkedList<int[]> charResult = new LinkedList<int[]>();
				int pos = posList[pointer];
		
				/* Suche des Zeichens im Chiffretext  */
				for (int j = 0; j < chiffretext.length; j++) 
				{
					if (chiffretext[j] == c) {
						if (pos < klartext.length)						  
							/*
						   * Die Paare gehen von 1-n, 
						   * das Array von 0-(n-1)
						   */
							charResult.add(new int[] { (pos + 1), (j + 1) }); 
					}
				}		
				if (charResult.size() > 0) {
					LinkedList<int[][]> newresult = combinate(result, charResult);
					result = newresult;
				}
				pointer++;
			}

		}
		return result;
	}

	private static LinkedList<int[][]> combinate(LinkedList<int[][]> liste1,LinkedList<int[]> pairs) {
		
		LinkedList<int[][]> newResult = new LinkedList<int[][]>();
		if (liste1.size() == 0) {
			for (int[] e : pairs) {
				int[][] a = new int[1][2];
				a[0][0] = e[0];
				a[0][1] = e[1];
				newResult.add(a);
			}
		} else {
			for (int[][] e1 : liste1) {
				for (int[] e2 : pairs) {
					int[][] a = new int[e1.length + 1][2];
					for (int i = 0; i < e1.length; i++) {
						a[i][0] = e1[i][0];
						a[i][1] = e1[i][1];
					}
					a[e1.length][0] = e2[0];
					a[e1.length][1] = e2[1];
					newResult.add(a);
				}
			}

		}
		return newResult;

	}

	public int getPlen() {
		return klartext.length;
	}

	public int getClen() {
		return chiffretext.length;
	}

	public void reset() {
		this.pointer = 0;
	}
	
	public int getDelta()
	{
		return delta;
	}
}
