package allgemein.gui;

import java.util.ArrayList;
import java.util.LinkedList;

import allgemein.core.Calculator;
import allgemein.core.CalculatorListener;
import allgemein.core.CalculatorState;
import allgemein.core.Key;
import allgemein.core.KeyList;
import allgemein.core.PairGenerator;

public class DoppelwuerfelModel implements CalculatorListener
{

	private String plaintext;
	private String ciphertext;
	
	private int k1=1;
	private int k2=1;
	
	private int k1End=15;
	private int k2End=15;
	
	private int miss;
	private int delta;
	

	private boolean running;
	private boolean searchKey=false;
	
	private long time;
	
	private ArrayList<DoppelwuerfelListener> listener;
	private ArrayList<String> statusText;
	private DoppelwuerfelThread dwt;
	private LinkedList<Object[]> resultTable;
	
	public DoppelwuerfelModel()
	{
		 listener = new ArrayList<DoppelwuerfelListener>();
		 statusText= new ArrayList<String>();
		 miss=3;
	}
	
	
	public int getK1End() {
		return k1End;
	}


	public void setK1End(int k1End) {
		this.k1End = k1End;
	}


	public int getK2End() {
		return k2End;
	}


	public void setK2End(int k2End) {
		this.k2End = k2End;
	}


	public String getStatus(int i)
	{
		return statusText.get(i);
	}
	
	public int getStatusSize()
	{
		return statusText.size();
	}
	
	public boolean isSearchKey() {
		return searchKey;
	}


	public void setSearchKey(boolean searchKey) {
		this.searchKey = searchKey;
	}


	public void addListener(DoppelwuerfelListener dwl)
	{
		listener.add(dwl);
		modelChanged();
	}
	
	public int getMiss() {
		return miss;
	}

	public void setMiss(int miss) {
		this.miss = miss;
	}
	
	public void setK1(int k1)
	{
		this.k1=k1;
	}
	
	public void setK2(int k2)
	{
		this.k2=k2;
	}

	public int getK1()
	{
		return k1;
	}
	
	public int getK2()
	{
		return k2;
	}
	
	public void start()
	{
		running=true;
		resultTable = new LinkedList<Object[]>();
		clearStatusText();
		time = System.currentTimeMillis();
		PairGenerator pg = new PairGenerator(plaintext,ciphertext,delta);
		Calculator c = new Calculator(pg);
		c.setMiss(miss);
		c.setK1len(k1);
		c.setK2len(k2);
		if (searchKey)
		{
			c.setK1lenEnd(k1End);
			c.setK2lenEnd(k2End);
			c.setSearchRange(true);
		}
		c.addListener(this);
		dwt = new DoppelwuerfelThread(this,c);
		addStatusText("Gestartet!");
		modelChanged();
	}
	
	@SuppressWarnings("deprecation")
	public void stop()
	{
		try{
		if (running)
		{
			dwt.interrupt();
			dwt.stop();
			dwt.interrupt();
			dwt.destroy();
		}
		} catch(Throwable e)
		{
		}
		addStatusText("Gestoppt!");
		running=false;
		modelChanged();
	}
	

	public String getPlaintext() {
		return plaintext;
	}



	public void setPlaintext(String plaintext) {
		this.plaintext = plaintext;
	}



	public String getCiphertext() {
		return ciphertext;
	}



	public void setCiphertext(String ciphertext) {
		this.ciphertext = ciphertext;
	}

	public boolean isRunning() {
		return running;
	}


	public void clearStatusText()
	{
		statusText.clear();
	}

	public void addStatusText(String t)
	{
		statusText.add(t);
		modelChanged();
	}
	
	private void modelChanged()
	{
		for (DoppelwuerfelListener l : listener)
			l.modelChanged();
	}


	@Override
	public void stateChanged(CalculatorState s) {
		if (!running) return;
		int state = s.getState();
		if (state==CalculatorState.NEW_ROUND)
		{
			int i = (Integer)s.getData("ROUND");
			addStatusText("Neue Runde: " + i);
		}
		
		if (state==CalculatorState.WORKLIST_CREATED)
		{
			int i = (Integer)s.getData("SIZE");
			addStatusText("Arbeitsliste erstellt: " + i);
		}
		
		if (state==CalculatorState.CALCULATION_NEW_RESULT)
		{
			int i = (Integer)s.getData("POS");
			int a = (Integer)s.getData("MAX");
			int r = (Integer)s.getData("ROUND");
			addStatusText("Ergebnisfortschritt fr Runde " + r + ": " + i+"/"+a);
		}
		
		if (state==CalculatorState.CALCULATION_THREADS_START)
		{
			addStatusText("Threads gestartet...");
		}
		
		if (state==CalculatorState.CALCULATION_THREADS_STOP)
		{
			addStatusText("...Threads beendet");
		}
		
		if (state==CalculatorState.CALCULATION_RESULTENTRIES)
		{
			int keys = (Integer)s.getData("KEYS");
			
			addStatusText("Berechnung ergaben " + keys + " Schlssel");
		}
		
		if (state==CalculatorState.CALCULATION_FINISHED_NORESULTS)
		{
			addStatusText("Keine Ergebnisse");
			

			
			long now = System.currentTimeMillis();
			double t = ((double)now - (double)(time))/1000;
			
			int k1 = (Integer)s.getData("K1");
			int k2 = (Integer)s.getData("K2");
			resultTable.add(new Object[]{t,k1,k2,"-"});
			
			
			addStatusText("Dauer: " + t+"s");
			time=now;
		}
		
		if (state==CalculatorState.CALCULATION_FINISHED_SUCCESS)
		{
			KeyList a= (KeyList)s.getData("KEYLIST");
			if (a.size()==1)
				addStatusText("1 Ergebnis gefunden:");
			else
				addStatusText(a.size() + " Ergebnisse gefunden:");
		
			for (Key k : a)
			{
				addStatusText(k.toString());
			}
			
			int k1 = (Integer)s.getData("K1");
			int k2 = (Integer)s.getData("K2");
			
			long now = System.currentTimeMillis();
			double t = ((double)now - (double)(time))/1000;
			
			resultTable.add(new Object[]{t,k1,k2,a.toString()});
			
			addStatusText("Dauer: " + t+"s");
			time=now;
		}
		
		if (state == CalculatorState.NEW_KEYLENGTH_SEARCH_ROUND)
		{
			int k1 = (Integer)s.getData("K1");
			int k2 = (Integer)s.getData("K2");
			addStatusText("Betrachte Schlssel " + k1 +"/"+k2);
		}
		
		if (state == CalculatorState.INVALID_KEYLENGTH)
		{
			addStatusText("Schlssellnge fehlerhaft");
		}
		
		if (state == CalculatorState.INVALID_RANGE)
		{
			addStatusText("Schlsselraum fehlerhaft");
		}
		
	
		
	}
	
	public synchronized void calculationFinished()
	{
		running=false;
		addStatusText("Berechnungen beendet");
	}
	public LinkedList<Object[]> getResultTable()
	{
		return this.resultTable;
	}
	
	public int getDelta() {
		return delta;
	}


	public void setDelta(int delta) {
		this.delta = delta;
	}
	
}
